"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var Core =
/*#__PURE__*/
function () {
  function Core() {
    _classCallCheck(this, Core);

    this._runningPtext;
    this._runningFileId;
    this._callbacks;
    this._interval;
    this._remaining;
    this._isPaused;
  }

  _createClass(Core, [{
    key: "findAndStartTimer",
    value: function findAndStartTimer(text, fileId) {
      var _this = this;

      var callbacks = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      this._callbacks = callbacks;

      var ptext = this._findPomodoroText(text, fileId);

      if (this._runningPtext && ptext.operator === this._runningPtext.operator && ptext.time === this._runningPtext.time && ptext.extraTime === this._runningPtext.extraTime && ptext.content === this._runningPtext.content && this._runningFileId === fileId) {
        return;
      }

      if (ptext) {
        var intervalCallback = function intervalCallback(remaining) {
          _this._remaining = remaining;
          callbacks.interval && callbacks.interval(remaining, ptext);
        };

        if (!this._runningPtext || ptext.time !== this._runningPtext.time || ptext.content !== this._runningPtext.content || this._runningFileId !== fileId) {
          if (this._interval) {
            clearInterval(this._interval);
            this._interval = null;
            callbacks.cancel && callbacks.cancel();
          }

          var time = ptext.time + (ptext.extraTime || 0);
          this._remaining = time;
          callbacks.start && callbacks.start(ptext);

          this._startTimer(time, intervalCallback).then(function () {
            return callbacks.finish && callbacks.finish(ptext);
          });
        } else if (this._runningPtext && ptext.extraTime !== this._runningPtext.extraTime) {
          clearInterval(this._interval);

          var _time = this._remaining + (ptext.extraTime || 0) - (this._runningPtext.extraTime || 0);

          if (_time > 0) {
            this._startTimer(_time, intervalCallback).then(function () {
              return callbacks.finish && callbacks.finish(ptext);
            });
          } else {
            intervalCallback(0);
            callbacks.finish && callbacks.finish(ptext);
          }
        }

        this._runningPtext = ptext;
        this._runningFileId = fileId;

        if (ptext.operator === '-') {
          this._isPaused = true;
          return;
        } else if (this._isPaused) {
          this._isPaused = false;
          return;
        }
      } else {
        this._runningPtext = null;

        if (!this._runningFileId || fileId === this._runningFileId) {
          this._clearTimer();

          callbacks.cancel && callbacks.cancel();
        }
      }
    }
  }, {
    key: "_findPomodoroText",
    value: function _findPomodoroText(text, id) {
      var lines = text.split('\n');
      var lineNumber = 0;
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = lines[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var line = _step.value;
          var found = line.match(/(?:^|^ *(?:-|\*) |^ *(?:-|\*) \[ \] )\[(-|)p([0-9]+)(\+[0-9]+|)\] *(.+)/);

          if (found) {
            if (parseInt(found[2]) <= 0) continue;
            var checkboxRegex = /^ *(?:-|\*) \[/g;
            var checkboxCh = checkboxRegex.test(line) ? checkboxRegex.lastIndex : 0;
            var extraTimeRegex = /\[(?:-|)p[0-9]+/g;
            var extraTimeCh = extraTimeRegex.test(line) ? extraTimeRegex.lastIndex : 0;
            var extraTime = found[3] ? parseInt(found[3]) * 60 : 0;
            return {
              id: id,
              line: lineNumber,
              checkboxCh: checkboxCh,
              extraTimeCh: extraTimeCh,
              operator: found[1],
              time: parseInt(found[2]) * 60,
              extraTime: extraTime,
              content: found[4]
            };
          }

          lineNumber++;
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return false;
    }
  }, {
    key: "_startTimer",
    value: function _startTimer(timeSec, callback) {
      var _this2 = this;

      return new Promise(function (resolve) {
        _this2._interval = setInterval(function () {
          if (_this2._isPaused) return;
          callback(--timeSec);

          if (timeSec <= 0) {
            clearInterval(_this2._interval);
            _this2._interval = null;
            resolve();
          }
        }, 1000);
      });
    }
  }, {
    key: "_clearTimer",
    value: function _clearTimer() {
      clearInterval(this._interval);
      this._interval = null;
      this._remaining = 0;
      this._isPaused = false;
    }
  }, {
    key: "stopTimer",
    value: function stopTimer() {
      if (this._runningPtext == null) return;
      this._runningPtext = null;

      this._clearTimer();

      this._callbacks.cancel && this._callbacks.cancel();
    }
  }, {
    key: "retryLatest",
    value: function retryLatest() {
      var _this3 = this;

      if (!this._runningPtext) return;

      this._clearTimer();

      this._callbacks.start && this._callbacks.start(this._runningPtext);

      var intervalCallback = function intervalCallback(remaining) {
        _this3._remaining = remaining;
        _this3._callbacks.interval && _this3._callbacks.interval(remaining, _this3._runningPtext);
      };

      var time = this._runningPtext.time + (this._runningPtext.extraTime || 0);
      this._remaining = time;

      this._startTimer(time, intervalCallback).then(function () {
        return _this3._callbacks.finish && _this3._callbacks.finish(_this3._runningPtext);
      });

      if (this._runningPtext.operator === '-') {
        this._isPaused = true;
      }
    }
  }]);

  return Core;
}();

exports.default = Core;